// $Id: glade--.cc,v 1.25 1999/01/18 17:57:32 christof Exp $
/*  glade--: C++ frontend for glade (Gtk+ User Interface Builder)
 *  Copyright (C) 1998  Christof Petig
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include "TagStream.hh"
#include "Cxx.hh"
#include "Configuration.hh"
#include <unistd.h>
#include <cstdlib>
#include <sys/stat.h> // for makedir
#include <getopt.h>

struct Configuration Configuration;

enum longopts { FULLTABLE=0x100, GTKVERSION, GTKMMVERSION, AUTOCONF,
	GTK_OSTREAM };

const static struct option options[]=
{ { "directory",	required_argument, NULL, 'd' },
  { "name",		required_argument, NULL, 'm' },
  { "version",		no_argument,	   NULL, 'V' },
  { "debug",		no_argument,	   NULL, 'g' },
  { "pointers",		no_argument,	   NULL, 'p' },
  { "samplecode",       no_argument,       NULL, 's' },
  { "barebones",        no_argument,       NULL, 'r' },
  { "gladeonly",        no_argument,       NULL, 'r' },
  { "noautoconf",       no_argument,       NULL, 'A' },
  { "autoconf",         no_argument,       NULL, AUTOCONF },
  { "widgettable",      no_argument,       NULL, 'w' },
  { "fulltable",        no_argument,       NULL, FULLTABLE },
  { "help",		no_argument,       NULL, '?' },
  { "gtkversion",	required_argument, NULL, GTKVERSION },
  { "gtkmmversion",	required_argument, NULL, GTKMMVERSION },
  { "gtkostream",	no_argument,	   NULL, GTK_OSTREAM },
  { NULL,		0,		   NULL, 0 }
};

static bool parse_version(const char *arg,Gtk_Version &v)
{  int major=0,minor=0,micro=0;
   const char *nextp;
   major=strtol(arg,&nextp,10);
   if (!*nextp) { v=Gtk_Version(major,minor,micro); return true; }
   if (*nextp!='.' && *nextp!=',')
   {  cerr << "error parsing version string '" << arg << "\n";
      return false;
   }
   minor=strtol(nextp+1,&nextp,10);
   if (!*nextp) { v=Gtk_Version(major,minor,micro); return true; }
   if (*nextp!='.' && *nextp!=',')
   {  cerr << "error parsing version string '" << arg << "\n";
      return false;
   }
   micro=strtol(nextp+1,&nextp,10);
   if (!*nextp) { v=Gtk_Version(major,minor,micro); return true; }
   cerr << "error parsing version string '" << arg << "\n";
   return false;
}

int main(int argc,char **argv)
{  int opt;
   for (int i=0;i<argc;i++)
   {  if (i) Configuration.commandline+=' ';
      Configuration.commandline+=argv[i];
   }
   while ((opt=getopt_long(argc,argv,"d:m:Vgprs1Aw",options,NULL))!=EOF) 
    switch(opt)
   {  case 'd': Configuration.destination_directory=optarg;
         if (access(Configuration.destination_directory.c_str(),F_OK))
            if (mkdir(Configuration.destination_directory.c_str(),0755))
            {  perror(Configuration.destination_directory.c_str());
               return 2;
            }
         break;
      case 'm': Configuration.main_filename=optarg;
         break;
      case 'V': cout<< "glademm V"VERSION" (glade to Gtk-- converter)\n";
         return 0;
         break;
      case 'g': Configuration.debug=true;
         break;
      case 'p': Configuration.all_pointer=true;
         break;
      case 's': Configuration.sample_code=true;
         break;
      case 'r': Configuration.bare_bones=true;
         break;
      case 'A': Configuration.no_autoconf=true;
         break;
      case 'w': Configuration.lookup_table++;
         break;
      case FULLTABLE: Configuration.lookup_table+=2;
         break;
      case GTKVERSION: 
         {  Gtk_Version v;
            if (parse_version(optarg,v))
               Configuration.gtk_version=v;
            else return 1;
         }
         break;
      case GTKMMVERSION: 
         {  Gtkmm_Version v;
            if (parse_version(optarg,v))
               Configuration.gtkmm_version=v;
            else return 1;
         }
         break;
      case GTK_OSTREAM: Configuration.gtk_ostream=true;
         break;
      case '?': usage:
      	 cerr << "USAGE: \n" << argv[0] << " [<option> ...] <inputfile>\n"
      	     "options:\n"
      	     "\t--directory <destdir>\twrite files to different directory [RECOMMENDED]\n"
      	     "\t--name <name>\tmain and executable name\n"
      	     "\t--version""\tprint version\n"
      	     "\t--debug""\tdebug tag structure [INTERNAL]\n"
      	     "\t--pointers""\tuse pointer for all widgets [RECOMMENDED]\n"
      	     "\t--gladeonly""\tonly create *_glade.cc/hh files\n"
      	     "\t--samplecode""\tinclude sample code into .cc file\n"
      	     "\t--gtkversion m.n.c\tuse gtk-m.n's features\n"
      	     "\t--gtkmmversion m.n.c\tgenerate programs for gtkmm-m.n\n"
	     "\t--noautoconf""\tcreate Makefile (no autoconf/-make) [RECOMMENDED, see BUGS]\n"
	     "\t--widgettable""\tcreate widget lookup table for user named widgets [RECOMMENDED]\n"
	     "\t--fulltable""\tadd default name widgets to lookup table\n"
      	     ;
         return 1;
   }
   if (argc-optind!=1) goto usage;
   Configuration.in_filename=argv[optind];
   
   cout << "Generating code for gtk "
        << Configuration.gtk_version.major << '.'
	<< Configuration.gtk_version.minor << '.'
	<< Configuration.gtk_version.micro <<
	", gtkmm " << Configuration.gtkmm_version.major << '.'
	<< Configuration.gtkmm_version.minor << '.'
	<< Configuration.gtkmm_version.micro << '\n';
   Tag top("");
   {  TagStream t(Configuration.in_filename);
      t.load_project_file(&top);
   }
   Cxx *cxx=new Cxx(&top);
   cxx->WriteTags();
   delete cxx;
}
